@@ -172,6 +172,9 @@ class ObjectDB:
 
         from_paths, to_oids = zip(*to_add)
         jobs: Optional[int] = kwargs.get("batch_size", kwargs.get("jobs"))
+
+        callback.set_size(len(from_paths))
+
         generic.transfer(
             fs,
             list(from_paths),
@@ -203,17 +206,22 @@ class ObjectDB:
         jobs: Optional[int] = None,
     ) -> Iterator[str]:
         if prefixes:
-            paths: Union[str, List[str]] = list(
-                map(self.oid_to_path, prefixes)
-            )
+            paths: Union[str, List[str]] = list(map(self.oid_to_path, prefixes))
             if len(paths) == 1:
                 paths = paths[0]
+            prefix = True
         else:
             paths = self.path
-        yield from self.fs.find(paths, batch_size=jobs)
+            prefix = False
+        yield from self.fs.find(paths, batch_size=jobs, prefix=prefix)
 
     def path_to_oid(self, path) -> str:
-        parts = self.fs.path.parts(path)[-2:]
+        if self.fs.path.isabs(path):
+            self_path = self.fs.path.abspath(self.path)
+        else:
+            self_path = self.path
+        self_parts = self.fs.path.parts(self_path)
+        parts = self.fs.path.parts(path)[len(self_parts) :]
 
         if not (len(parts) == 2 and parts[0] and len(parts[0]) == 2):
             raise ValueError(f"Bad cache file path '{path}'")
@@ -234,9 +242,7 @@ class ObjectDB:
             try:
                 yield self.path_to_oid(path)
             except ValueError:
-                logger.debug(
-                    "'%s' doesn't look like a cache file, skipping", path
-                )
+                logger.debug("'%s' doesn't look like a cache file, skipping", path)
 
     def _oids_with_limit(
         self,
@@ -285,9 +291,7 @@ class ObjectDB:
                 yield oid
 
         if max_oids:
-            oids = self._oids_with_limit(
-                max_oids / total_prefixes, prefixes=[prefix]
-            )
+            oids = self._oids_with_limit(max_oids / total_prefixes, prefixes=[prefix])
         else:
             oids = self._list_oids(prefixes=[prefix])
 
@@ -346,12 +350,12 @@ class ObjectDB:
         remote_size, remote_oids = self._estimate_remote_size()
         return self._list_oids_traverse(remote_size, remote_oids, jobs=jobs)
 
-    def list_oids_exists(self, oids: List[str], jobs: Optional[int] = None):
+    def list_oids_exists(self, oids: Iterable[str], jobs: Optional[int] = None):
         """Return list of the specified oids which exist in this fs.
         Hashes will be queried individually.
         """
-        logger.debug(f"Querying {len(oids)} oids via object_exists")
         paths = list(map(self.oid_to_path, oids))
+        logger.debug(f"Querying {len(paths)} oids via object_exists")
         in_remote = self.fs.exists(paths, batch_size=jobs)
         yield from itertools.compress(oids, in_remote)
 
@@ -401,9 +405,7 @@ class ObjectDB:
         always_traverse = getattr(self.fs, "_ALWAYS_TRAVERSE", False)
 
         oids = set(oids)
-        if (
-            len(oids) == 1 or not self.fs.CAN_TRAVERSE
-        ) and not always_traverse:
+        if (len(oids) == 1 or not self.fs.CAN_TRAVERSE) and not always_traverse:
             remote_oids = self.list_oids_exists(oids, jobs=jobs)
             callback = partial(progress, "querying", len(oids))
             return list(wrap_iter(remote_oids, callback))
@@ -421,9 +423,7 @@ class ObjectDB:
         # From testing with S3, for remotes with 1M+ files, object_exists is
         # faster until len(oids) is at least 10k~100k
         if remote_size > self.fs.TRAVERSE_THRESHOLD_SIZE:
-            traverse_weight = (
-                traverse_pages * self.fs.TRAVERSE_WEIGHT_MULTIPLIER
-            )
+            traverse_weight = traverse_pages * self.fs.TRAVERSE_WEIGHT_MULTIPLIER
         else:
             traverse_weight = traverse_pages
         if len(oids) < traverse_weight and not always_traverse:
@@ -437,15 +437,11 @@ class ObjectDB:
             ret = list(oids & remote_oids)
             callback = partial(progress, "querying", len(remaining_oids))
             ret.extend(
-                wrap_iter(
-                    self.list_oids_exists(remaining_oids, jobs=jobs), callback
-                )
+                wrap_iter(self.list_oids_exists(remaining_oids, jobs=jobs), callback)
             )
             return ret
 
         logger.debug(f"Querying '{len(oids)}' oids via traverse")
-        remote_oids = self._list_oids_traverse(
-            remote_size, remote_oids, jobs=jobs
-        )
+        remote_oids = self._list_oids_traverse(remote_size, remote_oids, jobs=jobs)
         callback = partial(progress, "querying", remote_size)
         return list(oids & set(wrap_iter(remote_oids, callback)))
