@@ -28,7 +28,7 @@ class ThreadPoolExecutor(futures.ThreadPoolExecutor):
     _max_workers: int
 
     def __init__(
-        self, max_workers: int = None, cancel_on_error: bool = False, **kwargs
+        self, max_workers: Optional[int] = None, cancel_on_error: bool = False, **kwargs
     ):
         super().__init__(max_workers=max_workers, **kwargs)
         self._cancel_on_error = cancel_on_error
@@ -45,15 +45,18 @@ class ThreadPoolExecutor(futures.ThreadPoolExecutor):
         It does not create all the futures at once to reduce memory usage.
         """
 
+        it = zip(*iterables)
+        if self.max_workers == 1:
+            for args in it:
+                yield fn(*args)
+            return
+
         def create_taskset(n: int) -> Set[futures.Future]:
             return {self.submit(fn, *args) for args in islice(it, n)}
 
-        it = zip(*iterables)
         tasks = create_taskset(self.max_workers * 5)
         while tasks:
-            done, tasks = futures.wait(
-                tasks, return_when=futures.FIRST_COMPLETED
-            )
+            done, tasks = futures.wait(tasks, return_when=futures.FIRST_COMPLETED)
             for fut in done:
                 yield fut.result()
             tasks.update(create_taskset(len(done)))
@@ -64,26 +67,26 @@ class ThreadPoolExecutor(futures.ThreadPoolExecutor):
         if sys.version_info > (3, 9):
             # pylint: disable=unexpected-keyword-arg
             return super().shutdown(wait=wait, cancel_futures=cancel_futures)
+        else:
+            with self._shutdown_lock:
+                self._shutdown = True
+                if cancel_futures:
+                    # Drain all work items from the queue, and then cancel their
+                    # associated futures.
+                    while True:
+                        try:
+                            work_item = self._work_queue.get_nowait()
+                        except queue.Empty:
+                            break
+                        if work_item is not None:
+                            work_item.future.cancel()
 
-        with self._shutdown_lock:
-            self._shutdown = True
-            if cancel_futures:
-                # Drain all work items from the queue, and then cancel their
-                # associated futures.
-                while True:
-                    try:
-                        work_item = self._work_queue.get_nowait()
-                    except queue.Empty:
-                        break
-                    if work_item is not None:
-                        work_item.future.cancel()
-
-            # Send a wake-up to prevent threads calling
-            # _work_queue.get(block=True) from permanently blocking.
-            self._work_queue.put(None)
-        if wait:
-            for t in self._threads:
-                t.join()
+                # Send a wake-up to prevent threads calling
+                # _work_queue.get(block=True) from permanently blocking.
+                self._work_queue.put(None)  # type: ignore[arg-type]
+            if wait:
+                for t in self._threads:
+                    t.join()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         if self._cancel_on_error:
@@ -124,16 +127,20 @@ async def batch_coros(
     tasks = create_taskset(batch_size)
     results: Dict[int, Any] = {}
     while tasks:
-        done, _pending = await asyncio.wait(
+        done, pending = await asyncio.wait(
             tasks.keys(), timeout=timeout, return_when=asyncio.FIRST_COMPLETED
         )
         if not done and timeout:
+            for pending_fut in pending:
+                pending_fut.cancel()
             raise TimeoutError
         for fut in done:
             try:
                 result = fut.result()
             except Exception as exc:  # pylint: disable=broad-except
                 if not return_exceptions:
+                    for pending_fut in pending:
+                        pending_fut.cancel()
                     raise
                 result = exc
             index = tasks.pop(fut)
