@@ -15,13 +15,16 @@ from typing import (
     List,
     Literal,
     Optional,
+    Tuple,
     Union,
     cast,
     overload,
 )
 
 from fsspec.asyn import get_loop
-from funcy import cached_property
+from funcy import once_per_args
+
+from dvc_objects.utils import cached_property
 
 from ..executors import ThreadPoolExecutor, batch_coros
 from .callbacks import DEFAULT_CALLBACK, Callback
@@ -57,6 +60,33 @@ class LinkError(OSError):
         )
 
 
+@once_per_args
+def check_required_version(
+    pkg: str, dist: str = "dvc_objects", log_level=logging.WARNING
+):
+    from importlib import metadata
+
+    from packaging.requirements import InvalidRequirement, Requirement
+
+    try:
+        reqs = {
+            r.name: r.specifier for r in map(Requirement, metadata.requires(dist) or [])
+        }
+        version = metadata.version(pkg)
+    except (metadata.PackageNotFoundError, InvalidRequirement):
+        return
+
+    specifier = reqs.get(pkg)
+    if specifier and version and version not in specifier:
+        logger.log(
+            log_level,
+            "'%s%s' is required, but you have %r installed which is incompatible.",
+            pkg,
+            specifier,
+            version,
+        )
+
+
 class FileSystem:
     sep = "/"
 
@@ -79,11 +109,15 @@ class FileSystem:
         self.jobs = kwargs.get("jobs") or self._JOBS
         self.hash_jobs = kwargs.get("checksum_jobs") or self.HASH_JOBS
         self._config = kwargs
-        self.fs_args: Dict[str, Any] = {"skip_instance_cache": True}
-        self.fs_args.update(self._prepare_credentials(**kwargs))
         if fs:
             self.fs = fs
 
+    @cached_property
+    def fs_args(self) -> Dict[str, Any]:
+        ret = {"skip_instance_cache": True}
+        ret.update(self._prepare_credentials(**self._config))
+        return ret
+
     @property
     def config(self) -> Dict[str, Any]:
         return self._config
@@ -114,7 +148,7 @@ class FileSystem:
 
     @cached_property
     def version_aware(self) -> bool:
-        return self.fs_args.get("version_aware", False)
+        return self._config.get("version_aware", False)
 
     @staticmethod
     def _get_kwargs_from_urls(urlpath: str) -> "Dict[str, Any]":
@@ -126,7 +160,8 @@ class FileSystem:
         return options
 
     def _prepare_credentials(
-        self, **config: Dict[str, Any]  # pylint: disable=unused-argument
+        self,
+        **config: Dict[str, Any],  # pylint: disable=unused-argument
     ) -> Dict[str, Any]:
         """Prepare the arguments for authentication to the
         host filesystem"""
@@ -134,20 +169,14 @@ class FileSystem:
 
     @classmethod
     def get_missing_deps(cls) -> List[str]:
-        import importlib
+        from importlib.util import find_spec
 
-        missing: List[str] = []
-        for package, module in cls.REQUIRES.items():
-            try:
-                importlib.import_module(module)
-            except ImportError:
-                missing.append(package)
-
-        return missing
+        return [pkg for pkg, mod in cls.REQUIRES.items() if not find_spec(mod)]
 
     def _check_requires(self, **kwargs):
         from .scheme import Schemes
 
+        check_required_version(pkg="fsspec")
         missing = self.get_missing_deps()
         if not missing:
             return
@@ -204,7 +233,7 @@ class FileSystem:
         path: AnyFSPath,
         offset: int,
         length: int,
-        delimiter: bytes = None,
+        delimiter: Optional[bytes] = None,
     ) -> bytes:
         return self.fs.read_block(path, offset, length, delimiter=delimiter)
 
@@ -215,27 +244,23 @@ class FileSystem:
         on_error: Literal["raise", "omit", "return"] = "raise",
         **kwargs: Any,
     ) -> Union[bytes, Dict[AnyFSPath, bytes]]:
-        return self.fs.cat(
-            path, recursive=recursive, on_error=on_error, **kwargs
-        )
+        return self.fs.cat(path, recursive=recursive, on_error=on_error, **kwargs)
 
     def cat_ranges(
         self,
         paths: List[AnyFSPath],
         starts: List[int],
         ends: List[int],
-        max_gap: int = None,
+        max_gap: Optional[int] = None,
         **kwargs,
     ) -> List[bytes]:
-        return self.fs.cat_ranges(
-            paths, starts, ends, max_gap=max_gap, **kwargs
-        )
+        return self.fs.cat_ranges(paths, starts, ends, max_gap=max_gap, **kwargs)
 
     def cat_file(
         self,
         path: AnyFSPath,
-        start: int = None,
-        end: int = None,
+        start: Optional[int] = None,
+        end: Optional[int] = None,
         **kwargs: Any,
     ) -> bytes:
         return self.fs.cat_file(path, start=start, end=end, **kwargs)
@@ -255,9 +280,9 @@ class FileSystem:
     def read_text(
         self,
         path: AnyFSPath,
-        encoding: str = None,
-        errors: str = None,
-        newline: str = None,
+        encoding: Optional[str] = None,
+        errors: Optional[str] = None,
+        newline: Optional[str] = None,
         **kwargs: Any,
     ) -> str:
         return self.fs.read_text(
@@ -268,9 +293,9 @@ class FileSystem:
         self,
         path: AnyFSPath,
         value: str,
-        encoding: str = None,
-        errors: str = None,
-        newline: str = None,
+        encoding: Optional[str] = None,
+        errors: Optional[str] = None,
+        newline: Optional[str] = None,
         **kwargs: Any,
     ) -> None:
         self.fs.write_text(
@@ -290,9 +315,7 @@ class FileSystem:
     ) -> None:
         return self.fs.pipe(path, value=value, **kwargs)
 
-    def touch(
-        self, path: AnyFSPath, truncate: bool = True, **kwargs: Any
-    ) -> None:
+    def touch(self, path: AnyFSPath, truncate: bool = True, **kwargs: Any) -> None:
         return self.fs.touch(path, truncate=truncate, **kwargs)
 
     def checksum(self, path: AnyFSPath) -> str:
@@ -302,9 +325,7 @@ class FileSystem:
         self.makedirs(self.path.parent(to_info))
         self.fs.copy(from_info, to_info)
 
-    def cp_file(
-        self, from_info: AnyFSPath, to_info: AnyFSPath, **kwargs: Any
-    ) -> None:
+    def cp_file(self, from_info: AnyFSPath, to_info: AnyFSPath, **kwargs: Any) -> None:
         self.fs.cp_file(from_info, to_info, **kwargs)
 
     @overload
@@ -351,7 +372,7 @@ class FileSystem:
             return fut.result()
         executor = ThreadPoolExecutor(max_workers=jobs, cancel_on_error=True)
         with executor:
-            return list(executor.map(self.fs.exists, path))
+            return list(executor.map(callback.wrap_fn(self.fs.exists), path))
 
     def lexists(self, path: AnyFSPath) -> bool:
         return self.fs.lexists(path)
@@ -394,15 +415,11 @@ class FileSystem:
         return not (self.is_symlink(path) or self.is_hardlink(path))
 
     @overload
-    def ls(
-        self, path: AnyFSPath, detail: Literal[True], **kwargs
-    ) -> "Iterator[Entry]":
+    def ls(self, path: AnyFSPath, detail: Literal[True], **kwargs) -> "Iterator[Entry]":
         ...
 
     @overload
-    def ls(
-        self, path: AnyFSPath, detail: Literal[False], **kwargs
-    ) -> Iterator[str]:
+    def ls(self, path: AnyFSPath, detail: Literal[False], **kwargs) -> Iterator[str]:
         ...
 
     def ls(self, path, detail=False, **kwargs):
@@ -440,9 +457,7 @@ class FileSystem:
             for result in executor.imap_unordered(find, path):
                 yield from result
 
-    def mv(
-        self, from_info: AnyFSPath, to_info: AnyFSPath, **kwargs: Any
-    ) -> None:
+    def mv(self, from_info: AnyFSPath, to_info: AnyFSPath, **kwargs: Any) -> None:
         self.fs.mv(from_info, to_info)
 
     move = mv
@@ -463,8 +478,48 @@ class FileSystem:
 
     remove = rm
 
-    def info(self, path: AnyFSPath, **kwargs) -> "Entry":
-        return self.fs.info(path, **kwargs)
+    @overload
+    def info(
+        self,
+        path: AnyFSPath,
+        callback: "Callback" = ...,
+        batch_size: Optional[int] = ...,
+        **kwargs,
+    ) -> "Entry":
+        ...
+
+    @overload
+    def info(
+        self,
+        path: List[AnyFSPath],
+        callback: "Callback" = ...,
+        batch_size: Optional[int] = ...,
+    ) -> List["Entry"]:
+        ...
+
+    def info(self, path, callback=DEFAULT_CALLBACK, batch_size=None, **kwargs):
+        if isinstance(path, str):
+            return self.fs.info(path, **kwargs)
+        callback.set_size(len(path))
+        jobs = batch_size or self.jobs
+        if self.fs.async_impl:
+            loop = get_loop()
+            fut = asyncio.run_coroutine_threadsafe(
+                batch_coros(
+                    [
+                        self.fs._info(p, **kwargs)  # pylint: disable=protected-access
+                        for p in path
+                    ],
+                    batch_size=jobs,
+                    callback=callback,
+                ),
+                loop,
+            )
+            return fut.result()
+        executor = ThreadPoolExecutor(max_workers=jobs, cancel_on_error=True)
+        with executor:
+            func = partial(self.fs.info, **kwargs)
+            return list(executor.map(callback.wrap_fn(func), path))
 
     def mkdir(
         self, path: AnyFSPath, create_parents: bool = True, **kwargs: Any
@@ -479,7 +534,7 @@ class FileSystem:
         from_file: Union[AnyFSPath, "BinaryIO"],
         to_info: AnyFSPath,
         callback: Callback = DEFAULT_CALLBACK,
-        size: int = None,
+        size: Optional[int] = None,
         **kwargs,
     ) -> None:
         if size:
@@ -489,9 +544,7 @@ class FileSystem:
             self.upload_fobj(stream, to_info, size=size)
         else:
             assert isinstance(from_file, str)
-            self.fs.put_file(
-                os.fspath(from_file), to_info, callback=callback, **kwargs
-            )
+            self.fs.put_file(os.fspath(from_file), to_info, callback=callback, **kwargs)
         self.fs.invalidate_cache(self.path.parent(to_info))
 
     def get_file(
@@ -528,7 +581,7 @@ class FileSystem:
         self,
         path: AnyFSPath,
         total: bool = True,
-        maxdepth: int = None,
+        maxdepth: Optional[int] = None,
         **kwargs: Any,
     ) -> Union[int, Dict[AnyFSPath, int]]:
         return self.fs.du(path, total=total, maxdepth=maxdepth, **kwargs)
@@ -539,7 +592,7 @@ class FileSystem:
         to_info: Union[AnyFSPath, List[AnyFSPath]],
         callback: "Callback" = DEFAULT_CALLBACK,
         recursive: bool = False,  # pylint: disable=unused-argument
-        batch_size: int = None,
+        batch_size: Optional[int] = None,
     ):
         jobs = batch_size or self.jobs
         if self.fs.async_impl:
@@ -567,7 +620,7 @@ class FileSystem:
         to_info: Union[AnyFSPath, List[AnyFSPath]],
         callback: "Callback" = DEFAULT_CALLBACK,
         recursive: bool = False,  # pylint: disable=unused-argument
-        batch_size: int = None,
+        batch_size: Optional[int] = None,
     ) -> None:
         # Currently, the implementation is non-recursive if the paths are
         # provided as a list, and recursive if it's a single path.
@@ -595,9 +648,7 @@ class FileSystem:
                 return localfs.makedirs(to_info, exist_ok=True)
 
             to_infos = [
-                localfs.path.join(
-                    to_info, *self.path.relparts(info, from_info)
-                )
+                localfs.path.join(to_info, *self.path.relparts(info, from_info))
                 for info in from_infos
             ]
 
@@ -624,9 +675,7 @@ class FileSystem:
     def modified(self, path: AnyFSPath) -> datetime.datetime:
         return self.fs.modified(path)
 
-    def sign(
-        self, path: AnyFSPath, expiration: int = 100, **kwargs: Any
-    ) -> str:
+    def sign(self, path: AnyFSPath, expiration: int = 100, **kwargs: Any) -> str:
         return self.fs.sign(path, expiration=expiration, **kwargs)
 
 
@@ -655,12 +704,43 @@ class ObjectFileSystem(FileSystem):  # pylint: disable=abstract-method
     ) -> Iterator[str]:
         if isinstance(path, str):
             paths = [path]
-        for path in paths:
-            if prefix:
-                with_prefix = self.path.parent(path)
-                yield from self.fs.find(
-                    with_prefix, prefix=self.path.parts(path)[-1]
-                )
-            else:
-                with_prefix = path
-                yield from self.fs.find(path)
+        else:
+            paths = path
+
+        def _make_args(paths: List[AnyFSPath]) -> Iterator[Tuple[str, str]]:
+            for path in paths:
+                if prefix and not path.endswith(self.path.flavour.sep):
+                    parent = self.path.parent(path)
+                    yield parent, self.path.parts(path)[-1]
+                else:
+                    yield path, ""
+
+        args = list(_make_args(paths))
+        if len(args) == 1:
+            path, prefix_str = args[0]
+            yield from self.fs.find(path, prefix=prefix_str)
+            return
+
+        jobs = batch_size or self.jobs
+        if self.fs.async_impl:
+            loop = get_loop()
+            fut = asyncio.run_coroutine_threadsafe(
+                batch_coros(
+                    [
+                        self.fs._find(  # pylint: disable=protected-access
+                            path, prefix=prefix_str
+                        )
+                        for path, prefix_str in args
+                    ],
+                    batch_size=jobs,
+                ),
+                loop,
+            )
+            for result in fut.result():
+                yield from result
+            return
+        # NOTE: this is not parallelized yet since imap_unordered does not
+        # handle kwargs. We do not actually support any non-async object
+        # storages, so this can be addressed when it is actually needed
+        for path, prefix_str in args:
+            yield from self.fs.find(path, prefix=prefix_str)
