@@ -11,7 +11,7 @@ from fsspec.asyn import get_loop
 from ..executors import ThreadPoolExecutor, batch_coros
 from .callbacks import DEFAULT_CALLBACK
 from .local import LocalFileSystem, localfs
-from .utils import as_atomic
+from .utils import as_atomic, umask
 
 if TYPE_CHECKING:
     from .base import AnyFSPath, FileSystem
@@ -20,9 +20,7 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-TransferErrorHandler = Callable[
-    ["AnyFSPath", "AnyFSPath", BaseException], None
-]
+TransferErrorHandler = Callable[["AnyFSPath", "AnyFSPath", BaseException], None]
 
 
 def log_exceptions(func: Callable) -> Callable:
@@ -62,6 +60,9 @@ def _link(
 
     func = getattr(to_fs, link)
     func(from_path, to_path)
+    if link == "reflink" and isinstance(from_fs, LocalFileSystem):
+        # NOTE: reflink may or may not clone src permissions
+        os.chmod(to_path, 0o666 & ~umask)
 
 
 def copy(
@@ -88,7 +89,7 @@ def copy(
             to_fs,
             to_path,
             callback=callback,
-            batch_size=batch_size,
+            batch_size=1 if isinstance(to_fs, LocalFileSystem) else batch_size,
             on_error=on_error,
         )
     if isinstance(to_fs, LocalFileSystem):
@@ -101,13 +102,19 @@ def copy(
             on_error=on_error,
         )
 
+    jobs = batch_size or to_fs.jobs
     put_file = callback.wrap_and_branch(to_fs.put_file)
+    put_file_kwargs = {}
+    if hasattr(to_fs.fs, "max_concurrency"):
+        put_file_kwargs["max_concurrency"] = jobs if len(from_path) == 1 else 1
 
     def _copy_one(from_p: "AnyFSPath", to_p: "AnyFSPath"):
         try:
             with from_fs.open(from_p, mode="rb") as fobj:
                 size = from_fs.size(from_p)
-                return put_file(fobj, to_p, size=size, callback=callback)
+                return put_file(
+                    fobj, to_p, size=size, callback=callback, **put_file_kwargs
+                )
         except Exception as exc:  # pylint: disable=broad-except
             if on_error is not None:
                 on_error(from_p, to_p, exc)
@@ -117,7 +124,6 @@ def copy(
     if len(from_path) == 1:
         return _copy_one(from_path[0], to_path[0])
 
-    jobs = batch_size or to_fs.jobs
     executor = ThreadPoolExecutor(max_workers=jobs, cancel_on_error=True)
     with executor:
         list(executor.imap_unordered(_copy_one, from_path, to_path))
@@ -131,11 +137,15 @@ def _put(
     batch_size: Optional[int] = None,
     on_error: Optional[TransferErrorHandler] = None,
 ) -> None:
+    jobs = batch_size or to_fs.jobs
     put_file = callback.wrap_and_branch(to_fs.put_file)
+    put_file_kwargs = {}
+    if hasattr(to_fs.fs, "max_concurrency"):
+        put_file_kwargs["max_concurrency"] = jobs if len(from_paths) == 1 else 1
 
     def _put_one(from_path: "AnyFSPath", to_path: "AnyFSPath"):
         try:
-            return put_file(from_path, to_path, callback=callback)
+            return put_file(from_path, to_path, callback=callback, **put_file_kwargs)
         except Exception as exc:  # pylint: disable=broad-except
             if on_error is not None:
                 on_error(from_path, to_path, exc)
@@ -145,7 +155,6 @@ def _put(
     if len(from_paths) == 1:
         return _put_one(from_paths[0], to_paths[0])
 
-    jobs = batch_size or to_fs.jobs
     if to_fs.fs.async_impl:
         put_coro = callback.wrap_and_branch_coro(
             to_fs.fs._put_file  # pylint: disable=protected-access
@@ -154,7 +163,7 @@ def _put(
         fut = asyncio.run_coroutine_threadsafe(
             batch_coros(
                 [
-                    put_coro(from_path, to_path, callback=callback)
+                    put_coro(from_path, to_path, callback=callback, **put_file_kwargs)
                     for from_path, to_path in zip(from_paths, to_paths)
                 ],
                 batch_size=jobs,
@@ -183,12 +192,18 @@ def _get(
     batch_size: Optional[int] = None,
     on_error: Optional[TransferErrorHandler] = None,
 ) -> None:
+    jobs = batch_size or from_fs.jobs
     get_file = callback.wrap_and_branch(from_fs.get_file)
+    get_file_kwargs = {}
+    if hasattr(from_fs.fs, "max_concurrency"):
+        get_file_kwargs["max_concurrency"] = jobs if len(from_paths) == 1 else 1
 
     def _get_one(from_path: "AnyFSPath", to_path: "AnyFSPath"):
         with as_atomic(localfs, to_path, create_parents=True) as tmp_file:
             try:
-                return get_file(from_path, tmp_file, callback=callback)
+                return get_file(
+                    from_path, tmp_file, callback=callback, **get_file_kwargs
+                )
             except Exception as exc:  # pylint: disable=broad-except
                 if on_error is not None:
                     on_error(from_path, to_path, exc)
@@ -198,7 +213,6 @@ def _get(
     if len(from_paths) == 1:
         return _get_one(from_paths[0], to_paths[0])
 
-    jobs = batch_size or from_fs.jobs
     if from_fs.fs.async_impl:
 
         async def _get_one_coro(from_path: "AnyFSPath", to_path: "AnyFSPath"):
@@ -206,7 +220,9 @@ def _get(
                 from_fs.fs._get_file  # pylint: disable=protected-access
             )
             with as_atomic(localfs, to_path, create_parents=True) as tmp_file:
-                return await get_coro(from_path, tmp_file, callback=callback)
+                return await get_coro(
+                    from_path, tmp_file, callback=callback, **get_file_kwargs
+                )
 
         loop = get_loop()
         fut = asyncio.run_coroutine_threadsafe(
@@ -240,13 +256,21 @@ def _try_links(
     to_fs: "FileSystem",
     to_path: "AnyFSPath",
     callback: "Callback" = DEFAULT_CALLBACK,
+    batch_size: Optional[int] = None,
 ) -> None:
     error = None
     while links:
         link = links[0]
 
         if link == "copy":
-            return copy(from_fs, from_path, to_fs, to_path, callback=callback)
+            return copy(
+                from_fs,
+                from_path,
+                to_fs,
+                to_path,
+                callback=callback,
+                batch_size=batch_size,
+            )
 
         try:
             _link(link, from_fs, from_path, to_fs, to_path)
@@ -258,15 +282,15 @@ def _try_links(
                 errno.ENOTSUP,
                 errno.EXDEV,
                 errno.ENOTTY,
+                errno.ENOSYS,
+                errno.EINVAL,
             ):
                 raise
             error = exc
 
         del links[0]
 
-    raise OSError(
-        errno.ENOTSUP, "no more link types left to try out"
-    ) from error
+    raise OSError(errno.ENOTSUP, "no more link types left to try out") from error
 
 
 def transfer(
@@ -292,12 +316,11 @@ def transfer(
     else:
         links = links or ["reflink", "copy"]
 
-    callback.set_size(len(from_path))
     # Try to link files sequentially. If/when the only remaining link type is
     # copy, the remaining copy operations will be batched.
     for i, (from_p, to_p) in enumerate(zip(from_path, to_path)):
-        if links[0] == "copy":
-            return copy(
+        if links and links[0] == "copy":
+            copy(
                 from_fs,
                 from_path[i:],
                 to_fs,
@@ -306,6 +329,7 @@ def transfer(
                 batch_size=batch_size,
                 on_error=on_error,
             )
+            return
         try:
             _try_links(
                 links,
@@ -314,6 +338,7 @@ def transfer(
                 to_fs,
                 to_p,
                 callback=callback,
+                batch_size=batch_size,
             )
         except OSError as exc:
             # If the target file already exists, we are going to simply
@@ -329,7 +354,7 @@ def transfer(
                 and isinstance(exc.__context__, FileExistsError)
             ):
                 logger.debug("'%s' file already exists, skipping", to_path)
-                return None
+                continue
 
             if on_error is not None:
                 on_error(from_p, to_p, exc)
