@@ -7,16 +7,7 @@ import sys
 import threading
 from concurrent import futures
 from contextlib import contextmanager, suppress
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Collection,
-    Dict,
-    Iterator,
-    Optional,
-    Set,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Collection, Dict, Iterator, Optional, Set, Union
 
 from ..executors import ThreadPoolExecutor
 from . import system
@@ -29,8 +20,11 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
+umask = os.umask(0)
+os.umask(umask)
 
 LOCAL_CHUNK_SIZE = 2**20  # 1 MB
+COPY_PBAR_MIN_SIZE = 2**30  # 1 GB
 
 
 def is_exec(mode: int) -> bool:
@@ -113,7 +107,7 @@ def remove(path: "AnyFSPath") -> None:
             raise
 
 
-def makedirs(path, exist_ok: bool = False, mode: int = None) -> None:
+def makedirs(path, exist_ok: bool = False, mode: Optional[int] = None) -> None:
     if mode is None:
         os.makedirs(path, exist_ok=exist_ok)
         return
@@ -150,45 +144,64 @@ def makedirs(path, exist_ok: bool = False, mode: int = None) -> None:
         )
 
 
+def _copyfile_with_pbar(
+    src: "AnyFSPath",
+    dest: "AnyFSPath",
+    total: int,
+    callback: Optional["Callback"] = None,
+    no_progress_bar: bool = False,
+    name: Optional[str] = None,
+):
+    from .callbacks import Callback
+
+    name = name if name else os.path.basename(dest)
+
+    if callback:
+        callback.set_size(total)
+
+    with open(src, "rb") as fsrc, open(dest, "wb+") as fdest:
+        with Callback.as_tqdm_callback(
+            callback,
+            size=total,
+            bytes=True,
+            disable=no_progress_bar,
+            desc=name,
+        ) as cb:
+            wrapped = cb.wrap_attr(fdest, "write")
+            while True:
+                buf = fsrc.read(LOCAL_CHUNK_SIZE)
+                if not buf:
+                    break
+                wrapped.write(buf)
+
+    if callback:
+        callback.absolute_update(total)
+
+
 def copyfile(
     src: "AnyFSPath",
     dest: "AnyFSPath",
-    callback: "Callback" = None,
-    no_progress_bar: bool = False,
-    name: str = None,
+    **kwargs,
 ) -> None:
     """Copy file with progress bar"""
-    name = name if name else os.path.basename(dest)
     total = os.stat(src).st_size
 
     if os.path.isdir(dest):
         dest = os.path.join(dest, os.path.basename(src))
 
-    if callback:
-        callback.set_size(total)
-
     try:
         system.reflink(src, dest)
+        # NOTE: reflink may or may not clone src permissions
+        os.chmod(dest, 0o666 & ~umask)
+        return
     except OSError:
-        from .callbacks import Callback
+        pass
 
-        with open(src, "rb") as fsrc, open(dest, "wb+") as fdest:
-            with Callback.as_tqdm_callback(
-                callback,
-                size=total,
-                bytes=True,
-                disable=no_progress_bar,
-                desc=name,
-            ) as cb:
-                wrapped = cb.wrap_attr(fdest, "write")
-                while True:
-                    buf = fsrc.read(LOCAL_CHUNK_SIZE)
-                    if not buf:
-                        break
-                    wrapped.write(buf)
+    if total < COPY_PBAR_MIN_SIZE:
+        shutil.copyfile(src, dest)
+        return
 
-    if callback:
-        callback.absolute_update(total)
+    _copyfile_with_pbar(src, dest, total, **kwargs)
 
 
 def tmp_fname(fname: "AnyFSPath" = "") -> "AnyFSPath":
