@@ -1,14 +1,13 @@
 import os
 from os import fspath
+from typing import Dict, List, Set, Tuple
 
 import pytest
 
 from dvc_objects.fs.local import LocalFileSystem
 
 
-@pytest.mark.parametrize(
-    "path, contents", [("file", "foo"), ("тест", "проверка")]
-)
+@pytest.mark.parametrize("path, contents", [("file", "foo"), ("тест", "проверка")])
 def test_local_fs_open(tmp_path, path, contents):
     file = tmp_path / path
     file.write_text(contents, encoding="utf8")
@@ -61,9 +60,7 @@ def test_local_fs_rm(tmp_path):
 
 
 def convert_to_sets(walk_results):
-    return [
-        (root, set(dirs), set(nondirs)) for root, dirs, nondirs in walk_results
-    ]
+    return [(root, set(dirs), set(nondirs)) for root, dirs, nondirs in walk_results]
 
 
 @pytest.fixture
@@ -74,16 +71,13 @@ def dir_path(tmp_path):
         ("тест", "проверка"),
         (
             "code.py",
-            "import sys\nimport shutil\n"
-            "shutil.copyfile(sys.argv[1], sys.argv[2])",
+            "import sys\nimport shutil\n" "shutil.copyfile(sys.argv[1], sys.argv[2])",
         ),
     ]:
         (tmp_path / file).write_text(contents, encoding="utf8")
     (tmp_path / "data" / "sub").mkdir(parents=True)
     (tmp_path / "data" / "file").write_text("file", encoding="utf8")
-    (tmp_path / "data" / "sub" / "file").write_text(
-        "sub_file", encoding="utf8"
-    )
+    (tmp_path / "data" / "sub" / "file").write_text("sub_file", encoding="utf8")
     return tmp_path
 
 
@@ -104,9 +98,11 @@ def test_walk(dir_path):
 
 def test_walk_detail(dir_path):
     fs = LocalFileSystem()
-    walk_results = list(fs.walk(fspath(dir_path), detail=True))
+    walk_results: List[Tuple[str, Dict[str, Dict], Dict[str, Dict]]] = list(
+        fs.walk(fspath(dir_path), detail=True)
+    )
 
-    expected = [
+    expected: List[Tuple[str, Set[str], Set[str]]] = [
         (str(dir_path), {"data"}, {"code.py", "bar", "тест", "foo"}),
         (str(dir_path / "data"), {"sub"}, {"file"}),
         (str(dir_path / "data" / "sub"), set(), {"file"}),
